local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage", "GatewayManage")

function get_property(pk, dn, property_list)
    ret = utils_device.get_property_value(pk, dn, property_list)
    if (nil == ret) then
        return nil
    end

    list = {}
    for n, id in ipairs(property_list) do 
        item = {}
        item["Name"] = id
        item["Id"] = id
        item["Value"] = ret[id]
        item["Time"] = tostring(os.time())
        table.insert(list, item)
    end

    return list
end


if ("GET" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    pk = args["ProductKey"]
    dn = args["DeviceName"]
    property_list = args["PropertyList"]

    utils.check_pk(pk)
    utils.check_dn(dn)

    if (nil == property_list) then
        utils.error_exit(400, "invalid parameter")
    end

    -- transfer single string to table
    if ("string" == type(property_list)) then
        property_list = {property_list}
    end

    ret = get_property(pk, dn, property_list)
    -- if (nil == ret) then
    --     utils.error_exit(500)
    -- end

    res = {}
    res["PropertyList"] = ret
    res["ProductKey"] = pk
    res["DeviceName"] = dn
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)

elseif ("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    pk = args["ProductKey"]
    dn = args["DeviceName"]
    property_list = args["PropertyList"]
    
    if (not (pk and dn and property_list)) then
        utils.error_exit(400, "invalid parameter")
    end
    
    local new_property = ""
    for _, property in ipairs(property_list) do
        new_property = new_property.."\""..property["Id"].."\":"..property["Value"]..","
    end
    
    new_property = string.sub(new_property, 1, #new_property - 1) 
    
    utils.log_info("set property params: "..new_property)
    ret = utils_device.set_property_value(pk, dn, new_property)
    if (not ret) then
        utils.error_exit(500, "set data into device failed")
    end

    list = {}
    for _, property in ipairs(property_list) do
        table.insert(list, property["Id"])
    end

    ret = get_property(pk, dn, list)
    if (nil == ret) then
        utils.error_exit(500)
    end

    res = {}
    res["ProductKey"] = pk
    res["DeviceName"] = dn
    res["PropertyList"] = ret
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
