/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reversed.
 */

var util = require('util');
var DBusProxy = require("./DBusProxy.js");

function CloudProxy() {
  this.bridge = new DBusProxy('iot.bridge.mqtt', '/iot/bridge/mqtt', 'iot.bridge.mqtt');
}

CloudProxy.prototype.publish = function (topic, payload, qos, callback) {
  if (topic === undefined || topic === null || topic === '') {
    return callback(new Error(util.format('"topic" cannot be "%s".', topic)), null);
  }
  if (payload === undefined || payload === null) {
    return callback(new Error(util.format('"payload" cannot be "%s".', payload)), null);
  }
  this.bridge.callServiceAsync('publish', topic, JSON.stringify(payload), qos,
    function (err) {
      callback(err);
    });
};

CloudProxy.prototype.subscribe = function (topic, qos, callback) {
  if (topic === undefined || topic === null || topic === '') {
    return callback(new Error(util.format('"topic" cannot be "%s".', topic)), null);
  }

  if (qos !== 0 && qos !== 1) {
    return callback(new Error(util.format('"qos" can only be "0" or "1".', payload)), null);
  }

  this.bridge.callServiceAsync('subscribe', topic, 'iot.runtime.js', qos, function (err) {
    callback(err);
  });
};

CloudProxy.prototype.unsubscribe = function (topic, callback) {
  if (topic === undefined || topic === null || topic === '') {
    return callback(new Error(util.format('"topic" cannot be "%s".', topic)), null);
  }

  this.bridge.callServiceAsync('unsubscribe', topic, 'iot.runtime.js', function (err) {
    callback(err);
  });
};

var proxy;
CloudProxy.default = function () {
  if (!proxy) {
    proxy = new CloudProxy();
  }
  return proxy;
};

module.exports = CloudProxy;