
var util = require('util');
var EventEmitter = require('events').EventEmitter;

var addon = require("./build/Release/smarthome_addon.node");

var SmartHome = function() {
    var self = this;
    this.devices = [];

    this.addDevices = function(device) {
        self.devices.push(device);
    }

    this.handleBroadcast = function(info) {
        try {
            /**
             * SmartHome事件分发原则
             * 1.　设备属性改变、设备event发生时，在对应的设备上emit事件
             * 2.　设备上线、下线、通用消息，在SmartHome上emit事件
             */
            var obj = JSON.parse(info);
            if (obj.key === "properties_changed") {
                for (var device of self.devices) {
                    if (device.id === obj.id) {
                        device.emit(obj.key, obj.value);
                    }
                }
            } else if (obj.key === "device_online") {
                self.emit(obj.key, obj.value);
            } else if (obj.key === "device_offline") {
                self.emit(obj.key, obj.id);
            } else {
                self.emit(obj.key, obj.value);
            }
        } catch (e) {
            console.log("SmartHome.handleBroadcast failed. reason : ", e.message);
        }
    }

    addon.registerBroadcastMessageListener(this.handleBroadcast);
}

util.inherits(SmartHome, EventEmitter);

var mockdevices = [
    {
      "thingId": "device_456",
      "method": [
        {
          "name": "get",
          "args": [
            {"arg_name": "property_name", "type": "STRING", "direction": "in"},
            {"arg_name": "value", "type": "VARIANT", "direction": "out"}
          ]
        },
        {
          "name": "get_all",
          "args": [
            {"arg_name": "properties", "type": "DICT", "direction": "out"}
          ]
        },
        {
          "name": "set",
          "args": [
            {"arg_name": "property_name", "type": "STRING", "direction": "in"},
            {"arg_name": "value", "type": "STRING", "direction": "in"},
            {"arg_name": "result", "type": "INT", "direction": "out"},
            {"arg_name": "message", "type": "STRING", "direction": "out"}
          ]
        }
      ],
      "event":[
      ],
      "properties":[
        {"name": "OnOff", "type": "BOOL", "access": "readwrite", "unit":"xxx","max_value":"xxx","min_value":"xxx","description": "zzz"},
        {"name": "Volt", "type": "NUMBER", "access": "readwrite", "unit":"xxx","max_value":"xxx","min_value":"yyy","description": "zzz"}
      ]
    }
];

SmartHome.prototype.getDevices = function() {
    return mockdevices;
}

SmartHome.prototype.getProperty = function(thingsId, property) {
    return addon.getProperty(thingsId, property);
}

SmartHome.prototype.getAllProperties = function(thingsId) {
    return addon.getProperty(thingsId, "");
}

SmartHome.prototype.setProperty = function(thingsId, property, value) {
    return addon.setProperty(thingsId, property, value);
}

SmartHome.prototype.callMethod = function(thingsId, method, args) {
    return addon.callMethod(thingsId, method, args);
}

module.exports = new SmartHome();
