const log = require('./log.js');
const tmp = require('./thing/manager.js');

const TAG = '[MockDProxy]';

const CMP_WELLKNOWNAME_PREFIX = 'iot.device.id';
const DMP_WELLKNOWNAME = 'iot.dmp.dimu';
const DMP_ID = 'DMP';

function parstId(wellknowName) {
  if (!wellknowName) {
    throw new Error('parstId Error: invalid wellknowName');
  }

  if (wellknowName === DMP_WELLKNOWNAME) {
    return DMP_ID;
  } else if (wellknowName.startsWith(CMP_WELLKNOWNAME_PREFIX)) {
    return wellknowName.substr(CMP_WELLKNOWNAME_PREFIX.length);
  }
  throw new Error('parstId Error: invalid wellknowName');
}
/**
 * current dbus
 * @param wellknowName
 * @param objPath
 * @param iface
 * @constructor
 */
function MockDProxy(wellknowName, objPath, iface) {
  log.d(TAG, `new MockDProxy ==> wellknowName: ${wellknowName}, objPath:${objPath}, iface: ${iface}`);
}

/**
 * want to visit dbus
 * @param wellknowName  util.format("iot.device.id%s", id)
 * @param objPath       util.format("/iot/device/id%s", id)
 * @param iface         util.format("iot.device.id%s", id)
 * @param signal
 * @param args
 */
MockDProxy.prototype.sendSignal = function (wellknowName, objPath, iface, signal, ...args) {
  log.d(TAG, `sendSignal ==> wellknowName: ${wellknowName}, objPath:${objPath}, iface: ${iface}, signal: ${signal}, args: ${args}`);
};


MockDProxy.prototype.callService = function (wellknowName, objPath, iface, method, ...args) {
  log.d(TAG, `callService ==> wellknowName: ${wellknowName}, method: ${method}, args: ${JSON.stringify(args)}`);
  const id = parstId(wellknowName);
  var data = {};
  var success = false;
  if (id === DMP_ID) {
    if (method === 'getDeviceList') {
      const ids = tmp.getIds(...args);
      data.devNum = ids.length;
      data.devList = ids;
      success = true;
    } else if (method === 'getDeviceProfile') {
      data.profile = tmp.getProfile(...args);
      success = true;
    } else {
      log.e(TAG, `cannot find the MockDMP method: ${method}`);
    }
  } else if (id && method === 'callServices') {
    const handle = tmp.getService(id);
    if (handle) {
      data = handle(id, ...args);
      success = true;
    } else {
      log.e(TAG, `cannot find the service ==> id: ${id}`);
    }
  } else {
    log.e(TAG, `cannot find the MockCMP ==> id: ${id}`);
  }
  const ret = {
    message: 'success',
    code: (success ? 0 : -1),
    params: data,
  };
  // sleep(200);
  return JSON.stringify(ret);
};

MockDProxy.prototype.isServiceOnline = function () {
  return true;
}

function sleep(ms) {
  const endTime = ms + Date.now();
  while (true) {
    if (Date.now() > endTime) {
      return;
    }
  }
}

MockDProxy.prototype.callServiceAsync = function (wellknowName, objPath, iface, method, ...args) {
  log.d(TAG, `callServiceAsync ==> wellknowName: ${wellknowName}, method: ${method}`);
  const callback = args.pop();
  const val = this.callService(wellknowName, objPath, iface, method, ...args);
  const value = JSON.parse(val);

  var ret;
  const id = parstId(wellknowName);
  if (id === DMP_ID || args[0] === 'set' || args[0] === 'get') {
    ret = value;
  } else {
    ret = {
      message: 'successfully',
      code: 0,
      params: {
        message: value.message,
        code: value.code,
        data: value.params,
      },
    };
  }

  setTimeout(() => {
    if (typeof callback === 'function') {
      if (!val) {
        callback(new Error('service return undefined'), undefined);
      } else {
        callback(undefined, JSON.stringify(ret));
      }
    }
  }, 200);
};

module.exports = MockDProxy;
