const util = require('util');
const Tag = require('./utils/constant');
const valuecomparer = require('./comparer.js');
const log = require('./utils/log');
const TAG = "[EventTrigger]";


var EventTrigger = function(params, sid) {
    this.params = params;
    this.fullfilled = false;
    this.id = util.format("%s_%s", this.params[Tag.PRODUCTKEY], this.params[Tag.DEVICENAME]);
    this.sceneid = sid;
    this.eventCode = this.params[Tag.EventCode];
};

EventTrigger.prototype.trigger = function (info) {
    return false;
};

/* 有时间戳
{
    "id" : "123",
       "version":"1.0",
    "params" : {
               "value":{
               "Power" : "on",
               "WF" : "2"
               },
               "time":1524448722000
    },
       "method":"thing.event.{tsl.event.identifier}.post"
}
无时间戳:
{
    "id" : "123",
    },
       "method":"thing.event.{tsl.event.identifier}.post"
}
无时间戳:
{
    "id" : "123",
    "version":"1.0",
    "params" : {
        "Power" : "on",
        "WF" : "2"
    },
       "method":"thing.event.{dsl.event.identifier}.post"
}
*/

EventTrigger.prototype.eventHappen = function(info) {
    log.i(TAG, "trigger.id = " + info.id + ", this.id = " + this.id);
    if (info.id !== this.id || info.eventName !== this.eventCode) {
        log.d(TAG, 'not for me');
        return false;
    }

    var params = info.params;
    if (!params || Object.keys(params).length === 0) {
        log.i(TAG, 'params is empty');
        return true;
    }

    var payload;
    if (typeof params.value === 'object' && params.value !== null) {
        payload = params.value;
    } else {
        payload = params;
    }

    var myprop = this.params[Tag.PROPERTYNAME];
    var compare = this.params[Tag.COMPARETYPE];
    var checker = this.params[Tag.COMPAREVALUE];
    if(!myprop || (!compare && !checker)) {
        return true;
    }

    if (!payload.hasOwnProperty(myprop)) {
        log.w(TAG, 'no property: ' + myprop);
        return false;
    }
    var value = payload[myprop];
    log.i(TAG, "orig", JSON.stringify(payload), "target {", myprop, compare,  checker, "}");
    this.fullfilled = valuecomparer(compare, value, checker);
    log.i(TAG, "fullfilled = ", this.fullfilled);
    return this.fullfilled;
}

EventTrigger.prototype.start = function() {
    return true;
};

EventTrigger.prototype.stop = function() {
    return true;
};

module.exports = EventTrigger;
