const moment = require('moment-timezone');
const Tag = require('../utils/constant');
const log = require('../utils/log');
const util = require('util');
const proxy = require("./ws/index");
const valuecomparer = require('../comparer.js');
const TAG = "[Condition]";

var Condition = function () {

};

/**
 * "conditions":[
 {
     "uri":"condition/device/status",
     "params":{
         "productKey":"",
         "deviceName":"",
         "propertyName":"",
         "compareType":"",
         "compareValue":""
     }
 }
 ]
 * @param conditions
 */
Condition.prototype.isConditionsMeeted = function (conditions, callback) {
    if( !conditions || conditions.length === 0) return callback(null, true);
    var meet = false;
    var hasReturn = false;
    var cnt = 0;
    conditions.forEach(function (condi) {
        if(!condi.uri || !condi.params) {
            hasReturn = true;
            return callback("params error, condition = " + JSON.stringify(condi), false);
        }
        switch (condi.uri) {
            case Tag.DeviceCondition : {
                var id = condi.params.productKey + "_" + condi.params.deviceName;
                var key = condi.params.propertyName;
                proxy.getProperty(id, [key], function(err, params) {
                    var value = params[key];
                    var satisfy = false;
                    if (!err) {
                        log.i(TAG, "origin value : " + value + " compareType : " + condi.params.compareType + " compareValue : " + condi.params.compareValue);
                        satisfy = valuecomparer(condi.params.compareType, value, condi.params.compareValue) //compare, value, checker
                        log.i(TAG, "valuecomparer satisfy = " + satisfy);
                    }
                    if (!hasReturn && !satisfy) {
                        hasReturn = true;
                        return callback(err, satisfy);
                    }
                    cnt++;
                    if ( !hasReturn && cnt === conditions.length) {
                        meet = true;
                        hasReturn = true;
                        return callback(null, meet);
                    }
                });
                break;
            }
            case Tag.TimeCondition : {
                try {
                    var format = condi.params.format;
                    var timezone = condi.params.timezoneID;
                    log.i('Format: ' + format + ', beginDate: ' + condi.params.beginDate
                        + ', endDate: ' + condi.params.endDate + ', timezoneID: ' + timezone);
                    var beginMoment = parseTime(format, condi.params.beginDate, timezone);
                    var endMoment = parseTime(format, condi.params.endDate, timezone);
                    var now = moment().tz(timezone);
                    log.i('Parsed beginDate: ' + beginMoment.format() + ', endDate: '
                        + endMoment.format() + ', now: ' + now.format());
                    if(now.isSameOrAfter(beginMoment) && now.isSameOrBefore(endMoment)) {
                        cnt++;
                    } else {
                        hasReturn = true;
                        return callback(null, false);
                    }
                } catch (err) {
                    hasReturn = true;
                    return callback(err, false);
                }
                break;
            }
            default : {
                log.w(util.format("[IFTTT::Scene::isConditionsMeeted] uri match null "));
                hasReturn = true;
                return callback("isConditionsMeeted uri match null", false);
                break;
            }
        }
        if ( !hasReturn && cnt === conditions.length) {
            meet = true;
            hasReturn = true;
            return callback(null, meet);
        }
    });
};

var TIME_REGEXP = /^((((y{4})-)?(M{2})-)?(d{2})\s+)?HH:mm(:(s{2}))?$/;

function parseTime(format, timeString, timezone) {
    var results = format.match(TIME_REGEXP);
    if (!results) {
        throw new Error('Format "' + format + '" does not match pattern yyyy-MM-dd HH:mm:ss.');
    }
    // Convert our date/time format to moment date/time format.
    var momentFormat = 'HH:mm';
    if (results[8]) {
        momentFormat = momentFormat + ':ss';
    }
    if (results[6]) {
        momentFormat = 'DD' + ' ' + momentFormat;
    }
    if (results[5]) {
        momentFormat = 'MM-' + momentFormat;
    }
    if (results[4]) {
        momentFormat = 'YYYY-' + momentFormat;
    }
    log.i('Parse time: ' + JSON.stringify({
        format: momentFormat,
        time: timeString,
        timezone: timezone
    }));
    return moment.tz(timeString, momentFormat, timezone);
};

module.exports = new Condition();