/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reserved.
 */

'use strict';

const path = require('path');
const util = require('util');
const dgram = require('dgram');
const client = dgram.createSocket('udp4');
const port = 12345;
const ip = '127.0.0.1';

class Log {
  static init(subName) {
    this.subModuleName = subName;
  }


  /**
   * {
  "content_size":35,
  "pid":0,
  "severity":"INFO",   // typo: serverity to severity
  "function":"function-name",
  "line":0,
  "file":"",
  "content":"FC Invoke End RequestId: request-id",
  "date":"2018-10-11",
  "time":"15:49:05.837",
  "request_id":"requestid",
  "module_name":"FCEdge",
  "sub_module_name":"function-name",
  "version":"1.0",
  "msg_type":"log_msg"
  }
   **/

  static d() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('DEBUG', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }

  static i() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('INFO', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }

  static e() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('ERROR', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }

  /*最严重的错误*/
  static f() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('FATAL', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }

  static m() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('MILESTONE', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }

  static w() {
    var info = stackInfo();
    var method = info['method'];
    var file = info['file'];
    var line = info['line'];
    var str = util.format(...arguments);
    var logInfo = genLineLog('WARNING', method, line, file, str, this.subModuleName);
    client.send(Buffer.from(logInfo), port, ip);
  }
}

function stackInfo() {
  var stackReg = /at\s+(.*)\s+\((.*):(\d*):(\d*)\)/i;
  var stackReg2 = /at\s+()(.*):(\d*):(\d*)/i;
  var stacklist = (new Error()).stack.split('\n').slice(3);
  var s = stacklist[0];
  var sp = stackReg.exec(s) || stackReg2.exec(s);
  var data = {};
  if (sp && sp.length === 5) {
    data.method = sp[1];
    data.path = sp[2];
    data.line = parseInt(sp[3]);
    data.pos = sp[4];
    data.file = path.basename(data.path);
  }
  return data;
}

function genLineLog(severity, functionName, line, file, content, sub_module_name) {
  if (!sub_module_name) {
    sub_module_name = "";
  }
  var log = {};
  log.content_size = 0;
  log.module_name = 'ifttt';
  log.pid = process.pid;
  log.severity = severity;
  log.function = functionName;
  log.line = line;
  log.file = file;
  log.content = content;
  log.data = new Date().Format('yyyy-MM-dd');
  log.time = new Date().Format('HH:mm:ss:SS');
  log.request_id = 'requestid';
  log.sub_module_name = sub_module_name;
  log.version = '1.0';
  log.msg_type = 'log_msg';
  return JSON.stringify(log);
}

// 对Date的扩展，将 Date 转化为指定格式的String
// 月(M)、日(d)、小时(h)、分(m)、秒(s)、季度(q) 可以用 1-2 个占位符，
// 年(y)可以用 1-4 个占位符，毫秒(S)只能用 1 个占位符(是 1-3 位的数字)
// 例子：
// (new Date()).Format("yyyy-MM-dd hh:mm:ss.S") ==> 2006-07-02 08:09:04.423
// (new Date()).Format("yyyy-M-d h:m:s.S")      ==> 2006-7-2 8:9:4.18
Date.prototype.Format = function (fmt) {
  var o = {
    "M+": this.getMonth() + 1,                 //月份
    "d+": this.getDate(),                    //日
    "h+": this.getHours() % 12 == 0 ? 12 : this.getHours() % 12, //小时
    "H+": this.getHours(), //小时
    "m+": this.getMinutes(),                 //分
    "s+": this.getSeconds(),                 //秒
    "q+": Math.floor((this.getMonth() + 3) / 3), //季度
    "S": this.getMilliseconds()             //毫秒
  };
  if (/(y+)/.test(fmt))
    fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
  for (var k in o)
    if (new RegExp("(" + k + ")").test(fmt))
      fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
  return fmt;
};

module.exports = Log;
